#ifndef PROTOCOLMANAGER_H
#define	PROTOCOLMANAGER_H

#include <Response.pb.h>
#include <Request.pb.h>
#include <Configuration.h>
#include <DBManager.h>

#include <tango.h>

namespace MetadataImporter_ns
{

class MetadataImporter;

//Protocol buffer request class shared pointer
typedef boost::shared_ptr<Request> RequestSP;

//Protocol buffer response class shared pointer
typedef boost::shared_ptr<Response> ResponseSP;

class ProtocolManager : public Tango::LogAdapter
{
public:
//------------------------------------------------------------------------------
//  [Public] Shared pointer typedef
//------------------------------------------------------------------------------
    typedef boost::shared_ptr<ProtocolManager> SP;

protected:
//------------------------------------------------------------------------------
//  [Protected] Constructor destructor deleter
//------------------------------------------------------------------------------
    ProtocolManager(MetadataImporter*, Configuration::SP, DBManager::SP);

    virtual ~ProtocolManager();

    class Deleter;
    friend class Deleter;
    class Deleter
    {
        public:
            void operator()(ProtocolManager* d) { delete d; }
    };

public:
//------------------------------------------------------------------------------
//	[Public] Class creation method
//------------------------------------------------------------------------------
    static ProtocolManager::SP create(MetadataImporter*, Configuration::SP,
        DBManager::SP);

//------------------------------------------------------------------------------
//	[Public] Remote endpoint setter method
//------------------------------------------------------------------------------
    virtual void setRemoteEndpoint(std::string);

//------------------------------------------------------------------------------
//  [Public] Time between request method
//------------------------------------------------------------------------------
    virtual bool waitBeforeRequest();

//------------------------------------------------------------------------------
//  [Public] Reset protocol status method
//------------------------------------------------------------------------------
    virtual void resetProtocolStatus();

//------------------------------------------------------------------------------
//  [Public] Request response management method
//------------------------------------------------------------------------------
    virtual RequestSP createRequest()
        throw(std::runtime_error, std::out_of_range);

    virtual void processResponse(ResponseSP)
        throw(std::runtime_error, std::out_of_range);

protected:
//------------------------------------------------------------------------------
//  [Protected] Request specific methods
//------------------------------------------------------------------------------
    virtual RequestSP createAuthroisation()
        throw(std::runtime_error);

    virtual RequestSP createValidation()
        throw(std::runtime_error);

    virtual RequestSP createMetadata()
        throw(std::runtime_error, std::out_of_range);

//------------------------------------------------------------------------------
//  [Protected] Response specific methods
//------------------------------------------------------------------------------
    virtual void processAuthroisation(ResponseSP)
        throw(std::runtime_error);

    virtual void processValidation(ResponseSP)
        throw(std::runtime_error);

    virtual void processMetadata(ResponseSP)
        throw(std::runtime_error, std::out_of_range);

//------------------------------------------------------------------------------
//  [Protected] Class variables
//------------------------------------------------------------------------------
    //MetadataImporter class pointer
    MetadataImporter* m_metadataImporter_p;

    //Configuration parameters shared pointer
    Configuration::SP m_configuration_sp;

    //Database manger shared pointer
    DBManager::SP m_dBManager_sp;

    //Client is authorised
    bool m_isAuthorised;

    //Table is validated
    bool m_isValidated;

    //Server has more data in buffer
    bool m_hasMoreData;

    //Address and port of remote endpoint
    std::string m_remoteEndpoint;
};

}   //End of namespace

#endif	/* PROTOCOLMANAGER_H */

